"""Implementation of XBAmodel class.

Peter Schubert, HHU Duesseldorf, May 2022
"""

import os
import time
import sbmlxdf

from .xba_compartment import XbaCompartment
from .xba_function import XbaFunction
from .xba_parameter import XbaParameter
from .xba_reaction import XbaReaction
from .xba_species import XbaSpecies


class XbaModel:

    def __init__(self, sbml_file):
        if os.path.exists(sbml_file) is False:
            print(f'{sbml_file} does not exist')
            return
        print(f'loading: {sbml_file} (last modified: {time.ctime(os.path.getmtime(sbml_file))})')
        sbml_model = sbmlxdf.Model(sbml_file)
        if sbml_model.isModel is False:
            print(f'{sbml_file} seems not to be a valid SBML model')
            return
        model_dict = sbml_model.to_df()
#        self.model_dict = model_dict  ######### - can be removed
        self.compartments = {cid: XbaCompartment(row)
                             for cid, row in model_dict['compartments'].iterrows()}
        self.parameters = {pid: XbaParameter(row)
                           for pid, row in model_dict['parameters'].iterrows()}
        self.functions = {fid: XbaFunction(row, )
                          for fid, row in model_dict['funcDefs'].iterrows()}
        self.species = {sid: XbaSpecies(row)
                        for sid, row in model_dict['species'].iterrows()}
        self.reactions = {rid: XbaReaction(row, self.functions)
                          for rid, row in model_dict['reactions'].iterrows()}

        for r in self.reactions.values():
            r.set_enzyme(self.species)
            r.add_species_usage(self.species)
